function [M,W,B] = banded_mmse_gen(Hf,Q,SNR_dB,reg_par)

% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012

snr_lin     = 10^(0.1*SNR_dB);
sigma2_used = max(1/snr_lin,reg_par);
[~,col_Hf]  = size(Hf);
N_active    = col_Hf;
Hf_banded   = Hf .* toeplitz([ones(1,Q+1) zeros(1,N_active-Q-1)]);

X = sigma2_used * eye(N_active) + ( (Hf_banded') * Hf_banded );
M = X \ (Hf_banded');
R = chol(0.5*(X+X'));
U = R ./ repmat(diag(R),1,N_active);
W = U * M;
B = U - diag(diag(U));

% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
% 
% [2] L. Rugini, P. Banelli, and G. Leus, Low-complexity banded equalizers
%     for OFDM systems in Doppler spread channels, EURASIP J. Appl. Signal
%     Processing, vol. 2006, Article ID 67404, pp. 1-13, 2006.
