function [M,W,B] = banded_windowed_mmse_gen(Hf_win,window,N_vir_half,Q,SNR_dB,reg_par)

% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012

snr_lin     = 10^(0.1*SNR_dB);
sigma2_used = max(1/snr_lin,reg_par);
[~,N]       = size(Hf_win);
F           = 1/sqrt(N) * fft(eye(N));
C           = F * diag(window) * (F');
invC        = F * diag(window.^(-1)) * (F');
N_active    = N - 2*N_vir_half;
N_index     = (1+N_vir_half):(N_active+N_vir_half);
C_banded    = C .* toeplitz([ones(1,Q+1) zeros(1,N-Q-1)]);

% linear equalizer design
Hf_banded = Hf_win .* toeplitz([ones(1,Q+1) zeros(1,N-Q-1)]);
Hf_used   = Hf_banded(N_index,N_index);
C_used    = C_banded(N_index,1:N);
M         = (Hf_used') / ( Hf_used * Hf_used' + sigma2_used * (C_used*C_used'));

% decision-feedback equalizer design
Hf_unwin  = invC * Hf_win;
Hf_banded = Hf_unwin .* toeplitz([ones(1,Q+1) zeros(1,N-Q-1)]);
Hf_used   = Hf_banded(1:N,N_index);
X = sigma2_used * eye(N_active) + (Hf_used' * Hf_used);
R = chol(0.5*(X+X'));
D = diag(R);
U = R ./ repmat(D,1,N_active);
W = U * M;
B = U - diag(diag(U));

% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
% 
% [2] L. Rugini, P. Banelli, and G. Leus, Low-complexity banded equalizers
%     for OFDM systems in Doppler spread channels, EURASIP J. Appl. Signal
%     Processing, vol. 2006, Article ID 67404, pp. 1-13, 2006.
