function ber = ber_estim_qpsk_bdfe(r,fff,fbf,bit1,bit2)

% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012

[Nactive,Nblocks] = size(r);
hard_estimate     = zeros(Nactive,Nblocks);
bit1_estimate     = zeros(Nactive,Nblocks);
bit2_estimate     = zeros(Nactive,Nblocks);
soft_estimate1    = fff * r;
soft_estimate2    = soft_estimate1(Nactive,1:Nblocks);

bit1_estimate(Nactive,1:Nblocks) = round((sign(real(soft_estimate2))+1)/2);
bit2_estimate(Nactive,1:Nblocks) = round((sign(imag(soft_estimate2))+1)/2);
hard_estimate(Nactive,1:Nblocks) = 1/sqrt(2)*((2*bit1_estimate(Nactive,1:Nblocks)-1)+sqrt(-1)*(2*bit2_estimate(Nactive,1:Nblocks)-1));

for index_subcarrier = (Nactive-1):-1:1
    
    interference_previously_decoded           = fbf(index_subcarrier,:) * hard_estimate;
    soft_estimate2                            = soft_estimate1(index_subcarrier,1:Nblocks) - interference_previously_decoded;
    
    bit1_estimate(index_subcarrier,1:Nblocks) = round((sign(real(soft_estimate2))+1)/2);
    bit2_estimate(index_subcarrier,1:Nblocks) = round((sign(imag(soft_estimate2))+1)/2);
    hard_estimate(index_subcarrier,1:Nblocks) = 1/sqrt(2)*((2*bit1_estimate(index_subcarrier,1:Nblocks)-1)+sqrt(-1)*(2*bit2_estimate(index_subcarrier,1:Nblocks)-1));

end

ber = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2)));

% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
% 
% [2] L. Rugini, P. Banelli, and G. Leus, Low-complexity banded equalizers
%     for OFDM systems in Doppler spread channels, EURASIP J. Appl. Signal
%     Processing, vol. 2006, Article ID 67404, pp. 1-13, 2006.
