function window = mbae_soe_win_gen(N,fd,Q)

% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012

% unitary DFT matrix
F = 1/sqrt(N) * fft(eye(N)); 

% (time-domain) autocorrelation matrix of a normalized channel path
R = toeplitz(besselj(0,2*pi*fd/N*(0:(N-1)))); 

% matrix A defined in Eq. 34 of [2]
A = toeplitz([(2*Q+1)/N sin(pi/N*(2*Q+1)*(1:(N-1)))./(N*sin(pi/N*(1:(N-1))))]);  

% Hadamard product between R and A
M = R.*A;

% matrix \tilde{F} used in Eq. 32 of [2]
F_tilde = [F(:,1:(Q+1)),F(:,(N-Q+1):N)];

% eigenvalue decomposition
[V,L] = eig(F_tilde'*M*F_tilde); 

% index of the largest eigenvalue
[~,index_max_eigenvalue] = max(diag(L)); 

% largest eigenvector;
b = V(:,index_max_eigenvalue); 

% elimination of the imaginary part (to increase numerical precision)
window = real(F_tilde*b); 

% normalization
window = sqrt(N)/sqrt(sum(window.^2)) * window;

% (optional part, used only for graphical reasons)
% forcing the maximum of the absolute values to be positive
[~,index_max_abs_value] = max(abs(window)); 
if window(index_max_abs_value)<0
    window = -window;
end

% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
% 
% [2] L. Rugini, P. Banelli, and G. Leus, Low-complexity banded equalizers
%     for OFDM systems in Doppler spread channels, EURASIP J. Appl. Signal
%     Processing, vol. 2006, Article ID 67404, pp. 1-13, 2006.
